///////////////////////////////////////////////////////////////////////////////
// Borderline File Parser class
//
// Authors: R. V. de Almeira Jr and H. J. Bortolossi.
//
// Created: 06/12/2005 -- Initial design and implementation.
//
// Last update: 23/12/2005
///////////////////////////////////////////////////////////////////////////////
package earth;

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.io.BufferedReader;
import java.io.IOException;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.geom.PgPolygonSet;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;

public class BorderlineFileParser
{
   protected PgPolygonSet borderlineSet; // Stores all shorelines
   protected int indexMatrix = 0;  // Overall number of points
   protected int indexPolygon = 0;

   protected BufferedReader reader;

   ////////////////////////////////////////////////////////////////////////////
   // Class constructor
   ////////////////////////////////////////////////////////////////////////////
   public BorderlineFileParser(BufferedReader input_reader,
		               PgPolygonSet input_borderlineSet)
   {
      // Parses input data
      this.borderlineSet = input_borderlineSet;
      this.reader = input_reader;

      // Loads the borderline data from file
      this.loadBorderalLine();
   }; // End of borderline()

   ////////////////////////////////////////////////////////////////////////////
   // Parses an ordinary longitude/latitude line computing the 3D position
   // on the planet.
   ////////////////////////////////////////////////////////////////////////////
   private PdVector processLine(String line)
   {
      double longitude = 0.0;
      double latitude = 0.0;
      PdVector value;

      // Reads the longitude
      int firstIndex = 0;
      int lastIndex = line.indexOf(' ');
      latitude  =
             Double.parseDouble(line.substring(firstIndex, lastIndex).trim());

      // Reads the latitude
      firstIndex = lastIndex + 1;
      lastIndex = line.length();
      longitude =
             Double.parseDouble(line.substring(firstIndex, lastIndex).trim());

      value = getFixedPoint(latitude, longitude);

      return(value);
   }; // End of processLine()

   ////////////////////////////////////////////////////////////////////////////
   // Computes the 3D position of an object given its geographical coordinates
   ////////////////////////////////////////////////////////////////////////////
   protected PdVector getFixedPoint(double lat_in_deg, double lon_in_deg)
   {
      PdVector newCoords;
      double lon_in_rad; // Longitude in radians
      double lat_in_rad; // Latitude in radians
      double x, y, z; // 3D coordinates of the point

      // Converstion to radians
      lon_in_rad = lon_in_deg*Math.PI/180.0;
      lat_in_rad = lat_in_deg*Math.PI/180.0;

      x = Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
      y = Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
      z = Math.sin(lat_in_rad);

      newCoords = new PdVector(x, y, z);
      return(newCoords);
   }; // End of getFixedPoint()

   ////////////////////////////////////////////////////////////////////////////
   // Loads borderline data from file
   ////////////////////////////////////////////////////////////////////////////
   protected void loadBorderalLine()
   {
      int n_of_pol = 0;
      int cur_pol = -1;
      int cur_n_of_pts = 0; // Stores the total number of points for each
                            // polygon. Example: nPoints = 125.
      PiVector polygon_idx = new PiVector(); // Stores the points indexes of
                                             // the polygon.
      String cur_lin = "";

      try
      {
         // Reads the first line that contains the number of polygons
         // (borderlines).
         cur_lin = reader.readLine();
         n_of_pol = getParameterValue(cur_lin);
         borderlineSet.setNumPolygons(n_of_pol);

         while ( !(cur_lin = reader.readLine()).startsWith("[ENDFILE]"))
         {
            if (cur_lin.startsWith("[ENDPOLYGON]"))
            {
               borderlineSet.setPolygon(cur_pol, polygon_idx);
               polygon_idx = new PiVector();
               continue;
            };

            if (cur_lin.startsWith("nPoints ="))
            {
               cur_n_of_pts = getParameterValue(cur_lin);
               cur_pol++;
               this.borderlineSet.getPolygon(cur_pol).setSize(cur_n_of_pts);
               continue;
            };
            if (cur_lin.trim().equals("") || cur_lin.startsWith("#"))
            {
               continue;
            };

            // If the execution reaches this point is because the input line
            // gives the (longitude, latitude) coordinates of a point.
            // The method addVertex(PdVector vector) returns the index (int)
            // of the point added to the polygon
            polygon_idx.addEntry(borderlineSet.addVertex(processLine(cur_lin)));
          }; // End of while()

          reader.close();
      }
      catch (IOException exp)
      {
        System.out.println(exp.toString());
      };
   };

   ////////////////////////////////////////////////////////////////////////////
   // Parses an input line with the form "parameter = value" and returns
   // "value".
   ////////////////////////////////////////////////////////////////////////////
   protected int getParameterValue(String str)
   {
      int firstIndex = 0;
      int lastIndex = 0;
      int value = 0;

      firstIndex = str.indexOf('=') + 1;
      lastIndex = str.length();
      value = Integer.parseInt(str.substring(firstIndex, lastIndex).trim());

      return(value);
   }; // End of getParameterValue()
}; // End of BorderlineReader()


